﻿Imports System.Linq.Expressions
Imports System.Reflection

Public Class LambdaUtil

    ''' <summary>
    ''' String.ToLowerメソッド。
    ''' </summary>
    Private Shared ReadOnly ToLower As MethodInfo = _
      GetType(String).GetMethod("ToLower", Type.EmptyTypes)

    ''' <summary>
    ''' String.Containsメソッド。
    ''' </summary>
    Private Shared ReadOnly Contains As MethodInfo = _
      GetType(String).GetMethod("Contains")

    ''' <summary>
    ''' ラムダ式におけるパラメータを作成する。
    ''' </summary>
    ''' <param name="paramName">パラメータ名</param>
    ''' <returns>パラメータ式</returns>
    Public Shared Function GetStringParameterExpression(ByVal paramName As String) As ParameterExpression

        Return Expression.Parameter(GetType(String), paramName)

    End Function

    ''' <summary>
    ''' ラムダ式におけるパラメータ式の配列を作成する。
    ''' </summary>
    ''' <param name="parameters">複数のパラメータ式</param>
    ''' <returns>パラメータ式の配列</returns>
    Public Shared Function GetParameterExpressions(ByVal ParamArray parameters As ParameterExpression()) As ParameterExpression()

        Return parameters

    End Function

    ''' <summary>
    ''' 「x.Contains("keyword")」を条件OR演算子（OrElse）で連結しながら式を作成する。
    ''' </summary>
    ''' <param name="parameter">ラムダ式の左にあるパラメータ</param>
    ''' <param name="keyword">検索キーワード</param>
    ''' <param name="curBody">現在の「x.Contains("keyword")」の表現文。初回はnullを指定。2回目以降は前回の戻り値を指定。</param>
    ''' <returns>「x.Contains("keyword")」を||演算子で連結した式</returns>
    Public Shared Function GetContainsExpression(ByVal parameter As Expression, ByVal keyword As String, ByVal curBody As Expression) As Expression

        Dim keywordValue = Expression.Constant(keyword, GetType(String))
        Dim newBody = Expression.Call( _
          Expression.Call(parameter, ToLower), _
          Contains, _
          Expression.Call(keywordValue, ToLower))

        If Not curBody Is Nothing Then
            Return Expression.OrElse(curBody, newBody)
        End If

        Return newBody

    End Function

    ''' <summary>
    ''' 動的に作成したラムダ式から式ツリー型オブジェクトを取得する。
    ''' </summary>
    ''' <param name="parameters">パラメータ式の配列（＝ラムダ式の左辺）</param>
    ''' <param name="body">式／文（＝ラムダ式の右辺）</param>
    ''' <returns>式ツリー型オブジェクト</returns>
    Public Shared Function GetLambdaExpressionWhere(ByVal parameters As ParameterExpression(), ByVal body As Expression) As Expression(Of Func(Of String, Boolean))

        Return Expression.Lambda(Of Func(Of String, Boolean))(body, parameters)

    End Function

End Class
