﻿Imports System.Linq.Expressions

Module Module1

    Sub Main()

        ' クエリ対象となるデータソース
        Dim dataSource As String() = _
          {"apple", "orange", "peach", "melon", "grape"}

        ' 検索キーワードを3つ取得
        Console.WriteLine("指定した文字を含む文字列を検索します。")
        Dim info As ConsoleKeyInfo = Console.ReadKey()
        Dim character As String = info.KeyChar.ToString()
        Console.WriteLine("かlかoを含むものを検索。")
        Dim keywords As String() = {character, "l", "o"}

        ' メソッド・ベースのLINQ文（静的なラムダ式を利用）
        'Dim query = dataSource.AsQueryable(). _
        '  Where(Function(item) _
        '    item.ToLower().Contains(keywords(0).ToLower()) OrElse _
        '    item.ToLower().Contains(keywords(1).ToLower()) OrElse _
        '    item.ToLower().Contains(keywords(2).ToLower())). _
        '  OrderByDescending(Function(item) item)

        ' メソッド・ベースのLINQ文（動的な式ツリー型を利用）
        Dim query = dataSource.AsQueryable(). _
          Where(GetExpressionTreeWhere(keywords)). _
          OrderByDescending(Function(item) item)

        ' 検索された用語を出力
        For Each term As String In query
            Console.WriteLine(term)
        Next

        ' 出力例（「m」を入力した場合）：
        ' 指定した文字を含む文字列を検索します。
        ' mかlかoを含むものを検索。
        ' orange
        ' melon
        ' apple

        ' 出力を確認するために実行を止める
        Console.ReadKey()

    End Sub

    Private Function GetExpressionTreeWhere(ByVal keywords As String()) As Expression(Of Func(Of String, Boolean))

        ' 引数が適切でない場合には、例外を発行する
        Dim length As Integer = keywords.Length
        If length = 0 Then
            Throw New ArgumentException("キーワード指定なし。")
        End If

        ' ラムダ式における左辺のパラメータの名前
        ' （「Function(item) ……」の部分に対応）
        Const paramName As String = "item"

        ' ラムダ式の左辺を構成するパラメータ項目の1つを作成
        Dim parameter As ParameterExpression = _
          LambdaUtil.GetStringParameterExpression(paramName)

        ' ラムダ式の左辺であるパラメータ（全項目）を配列にまとめる
        ' （※ただし今回はパラメータ項目は1つしかない）
        Dim parameters As ParameterExpression() = _
          LambdaUtil.GetParameterExpressions(parameter)

        ' ラムダ式の右辺である式を組み立てる
        Dim body As Expression = Nothing

        For i As Integer = 0 To length - 1
            ' 「item.ToLower().Contains(keywords[0].ToLower())」
            ' という式を作成して、2つ目以降のキーワードは
            ' 条件OR演算子（OrElse）で連結する
            body = LambdaUtil.GetContainsExpression( _
              parameter, keywords(i), IIf(i = 0, Nothing, body))
        Next


        ' 最後に、動的に作成したラムダ式から式ツリーを取得する
        Return LambdaUtil.GetLambdaExpressionWhere(parameters, body)

    End Function

End Module
